﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectMonorail.Drawnings;
using ProjectMonorail.Exceptions;

namespace ProjectMonorail.CollectionGenericObjects;
public abstract class AbstractCompany
{
    /// <summary>
	/// Размер места (ширина)
	/// </summary>
	protected readonly int _placeSizeWidth = 180;

    /// <summary>
    /// Размер места (высота)
    /// </summary>
    protected readonly int _placeSizeHeight = 80;

    /// <summary>
    /// Ширина окна
    /// </summary>
    protected readonly int _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    protected readonly int _pictureHeight;

    /// <summary>
    /// Коллекция Локомотив
    /// </summary>
    protected ICollectionGenericObjects<DrawningLocomotive>? _collection = null;

    /// <summary>
    /// Вычисление максимального количества элементов, который можно разместить в окне
    /// </summary>
    //private int GetMaxCount => _pictureWidth * _pictureHeight / (_placeSizeWidth * _placeSizeHeight); - было
    private int GetMaxCount => (_pictureWidth / _placeSizeWidth) * (_pictureHeight / _placeSizeHeight);

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="picWidth">Ширина окна</param>
    /// <param name="picHeight">Высота окна</param>
    /// <param name="collection">Коллекция Локомотив</param>
    public AbstractCompany(int picWidth, int picHeight, ICollectionGenericObjects<DrawningLocomotive> collection)
    {
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = collection;
        collection.MaxCount = GetMaxCount;
    }

    /// <summary>
    /// Перегрузка оператора сложения для класса
    /// </summary>
    /// <param name="company">Компания</param>
    /// <param name="car">Добавляемый объект</param>
    /// <returns></returns>
    public static int operator +(AbstractCompany company, DrawningLocomotive locomotive)
    {
        return company._collection.Insert(locomotive, new DrawningLocomotiveEqutables());
    }
    /// <summary>
    /// Перегрузка оператора удаления для класса
    /// </summary>
    /// <param name="company">Компания</param>
    /// <param name="position">Номер удаляемого объекта</param>
    /// <returns></returns>
    public static DrawningLocomotive operator -(AbstractCompany company, int position)
    {
        return company._collection.Remove(position);
    }

    /// <summary>
    /// Получение случайного объекта из коллекции
    /// </summary>
    /// <returns></returns>
    public DrawningLocomotive? GetRandomObject()
    {
        Random rnd = new();
        try
        {
            return _collection?.Get(rnd.Next(GetMaxCount));
        }
        catch (ObjectNotFoundException)
        {
            return null;
        }
    }

    /// <summary>
    /// Вывод всей коллекции
    /// </summary>
    /// <returns></returns>
    public Bitmap? Show()
    {
        Bitmap bitmap = new(_pictureWidth, _pictureHeight);
        Graphics graphics = Graphics.FromImage(bitmap);
        DrawBackground(graphics);

        SetObjectsPosition();
        for (int i = 0; i < (_collection?.Count ?? 0); ++i)
        {
            try
            {
                DrawningLocomotive? obj = _collection?.Get(i);
                obj?.DrawTransport(graphics);
            }
            catch (ObjectNotFoundException)
            {
                continue;
            }
        }

        return bitmap;
    }
    /// <summary>
    /// Сортировка
    /// </summary>
    /// <param name="comparer">Сравнитель объектов</param>
    public void Sort(IComparer<DrawningLocomotive?> comparer) => _collection?.CollectionSort(comparer);

    /// <summary>
    /// Вывод заднего фона
    /// </summary>
    /// <param name="g"></param>
    protected abstract void DrawBackground(Graphics g);

    /// <summary>
    /// Расстановка объектов
    /// </summary>
    protected abstract void SetObjectsPosition();
}