﻿using ProjectMonorail.CollectionGenericObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectMonorail.Drawnings;
using ProjectMonorail.Exceptions;
using System.Collections;

public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    private T?[] _collection;
    public int Count => _collection.Length;
    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;

    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T? Get(int position)
    {
        if (position < 0 || position >= _collection.Length)
        {
            return null;
        }
        return _collection[position];
    }
    public int Insert(T obj, IEqualityComparer<DrawningLocomotive?>? comparer = null)
    {
        // TODO выброс ошибки, если такой объект есть в коллекции
        // TODO выброс ошибки, если переполнение
        for (int i = 0; i < Count; i++)
        {
            if (comparer.Equals((_collection[i] as DrawningLocomotive), (obj as DrawningLocomotive)))
            {
                throw new ObjectAlreadyInCollectionException(i);
            }
        }

        for (int i = 0; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }

        throw new CollectionOverflowException(Count);
    }

    public int Insert(T obj, int position, IEqualityComparer<DrawningLocomotive?>? comparer = null)
    {
        // TODO выброс ошибки, если выход за границы массива
        // TODO выброс ошибки, если такой объект есть в коллекции
        // TODO выброс ошибки, если переполнение
        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException(position);
        }

        for (int i = 0; i < Count; i++)
        {
            if (comparer.Equals((_collection[i] as DrawningLocomotive), (obj as DrawningLocomotive)))
            {
                throw new ObjectAlreadyInCollectionException(i);
            }
        }

        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }

        for (int i = position + 1; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }

        for (int i = position - 1; i >= 0; i--)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }

        throw new CollectionOverflowException(Count);
    }

    public T Remove(int position)
    {
        if (position < 0 || position >= _collection.Length)
        {
            throw new PositionOutOfCollectionException(position);
        }
        if (_collection[position] == null) throw new ObjectNotFoundException(position);

        T? obj = _collection[position];
        _collection[position] = null;

        return obj;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; i++)
        {
            yield return _collection[i];
        }
    }

    public void CollectionSort(IComparer<T?> comparer)
    {
        Array.Sort(_collection, comparer);
    }
}