﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectMonorail.Drawnings;
using ProjectMonorail.MovementStrategy;

namespace ProjectMonorail
{
    public partial class FormMonorail : Form
    {
        private DrawningLocomotive? _drawningLocomotive;

        public AbstractStrategy? _strategy;
        public DrawningLocomotive SetLocomotive
        {
            set
            {
                _drawningLocomotive = value;
                _drawningLocomotive.SetPictureSize(pictureBoxMonorail.Width, pictureBoxMonorail.Height);
                comboBoxStrategy.Enabled = true;
                _strategy = null;
                Draw();
            }
        }
        public FormMonorail()
        {
            InitializeComponent();
            _strategy = null;
        }

        private void Draw()
        {
            if (_drawningLocomotive == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningLocomotive.DrawTransport(gr);
            pictureBoxMonorail.Image = bmp;
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningLocomotive == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            bool result = false;
            switch (name)
            {
                case "buttonUp":
                    result =
                    _drawningLocomotive.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    result =
                    _drawningLocomotive.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    result =
                    _drawningLocomotive.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    result =
                    _drawningLocomotive.MoveTransport(DirectionType.Right);
                    break;
            }
            if (result)
            {
                Draw();
            }
        }

        private void buttonStrategyStep_Click(object sender, EventArgs e)
        {
            if (_drawningLocomotive == null)
            {
                return;
            }

            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new MoveableLocomotive(_drawningLocomotive), pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            }

            if (_strategy == null)
            {
                return;
            }

            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();

            if (_strategy.GetStatus() == StrategyStatus.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }
    }
}


